#!/usr/bin/python
# -*- coding: UTF-8 -*-
from subprocess import Popen, PIPE, STDOUT
from re import search, DOTALL
from xml.dom.minidom import parseString

from onesheet.ClassFileMetadata import FileMetadata
from abc import ABCMeta

class TimeBasedMetadata(FileMetadata):
    __metaclass__ = ABCMeta
    def __init__(self, filename):
        FileMetadata.__init__(self, filename)
        self.GET_XML_COMMAND = 'ffprobe -print_format xml -show_streams'.split()
        self.fileName = filename
        if self.format_type == 'audio' or self.format_type == 'video':
            command = self.GET_XML_COMMAND
            command.append(self.fileName)
            p = Popen(command, stdout=PIPE, stderr=STDOUT, bufsize=0)
            self.rawdata = p.communicate()[0]
            self.fileXML = str(search('(<\?xml).*(</ffprobe>)', self.rawdata, DOTALL).group(0))
            self.xmlDom = parseString(self.fileXML)

    @property
    def Rawdata(self):
        return self.rawdata


    @property
    def totalRunningTimeRaw(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "audio":
                    return float(stream.getAttribute("duration"))
        # print self.fileXML
    #     return self.___totalRunningTimeRaw
        pass

    @property
    def totalRunningTimeSMPTE(self):

        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "audio":
                    seconds = float(stream.getAttribute("duration"))#TODO create SMPTE TRT return
                    break

        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        return str(int(h)).zfill(2) + ":" + str(int(m)).zfill(2) + ":" + str(int(s)).zfill(2)
        pass

    @property
    def totalSeconds(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "audio":
                    return float(stream.getAttribute("duration"))
        pass




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from re import search
from onesheet.TimeBasedMetadata import TimeBasedMetadata
from abc import ABCMeta

class VideoMetadata(TimeBasedMetadata):
    __metaclass__ = ABCMeta

    def __init__(self, filename):
        TimeBasedMetadata.__init__(self, filename)


    def __convertFrameToNDF(self, aFrameNumber):
        pass

    def __convertFrameToDF(self, aFrameNumber):
        pass

    def __sizeofHuman(self, num):
        num = int(num)
        for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
            if num < 1024.0:
                return "%3.1f %s" % (num, x)
            num /= 1024.0

    @property
    def videoCodec(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return stream.getAttribute("codec_name")

    @property
    def videoCodecLongName(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return stream.getAttribute("codec_long_name")
    @property
    def videoCodecTagString(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return stream.getAttribute("codec_tag_string")


    @property
    def videoCodecTag(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return stream.getAttribute("codec_tag")

    @property
    def videoFrameRate(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    top = float(stream.getAttribute("r_frame_rate").split('/')[0])
                    bottom = float(stream.getAttribute("r_frame_rate").split('/')[1])

                    return top/bottom


    @property
    def videoColorSpace(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    if stream.getAttribute("color_space"):
                        return str(stream.getAttribute("color_space"))
        pass

    @property
    def videoColorDepth(self):
        # TODO Make videoColorDepth method useful
        for stream in self.xmlDom.getElementsByTagName('stream'):
            if stream.getAttribute("codec_type") == "video":
                if stream.getAttribute("codec_long_name") == "Uncompressed 4:2:2 10-bit":
                    return 10
                else:
                    return str(stream.getAttribute("bits_per_raw_sample"))


        pass

    @property
    def videoColorSampling(self):
        # TODO Make VideoColorSampling method useful
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    if stream.getAttribute("codec_long_name") == "Uncompressed 4:2:2 10-bit":
                        return '4:2:2'
                    else:
                        return 'unknown'
        pass

    @property
    def videoBitRate(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return int(stream.getAttribute("bit_rate"))

    @property
    def videoBitRateH(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return self.__sizeofHuman(int(stream.getAttribute("bit_rate")))+"/s"

    @property
    def videoResolution(self):
            for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    height = stream.getAttribute("height")
                    width = stream.getAttribute("width")
                    return width + " x " + height

    @property
    def videoResolutionHeight(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return int(stream.getAttribute("height"))

    @property
    def videoResolutionWidth(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return int(stream.getAttribute("width"))

    @property
    def videoAspectRatio(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "video":
                    return str(stream.getAttribute("display_aspect_ratio"))



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from onesheet.TimeBasedMetadata import TimeBasedMetadata
from abc import ABCMeta
AUDIO_CODECS = {"8svx_exp": "8SVX",
                    "8svx_fib": "8SVX",
                    "aac": "AAC",
                    "aac_latm": "AAC",
                    "ac3": "AC-3",
                    "ac3_fixed": "AC-3",
                    "adpcm_4xm": "ADPCM",
                    "adpcm_adx": "ADPCM",
                    "adpcm_afc": "ADPCM",
                    "adpcm_ct": "ADPCM",
                    "adpcm_dtk": "ADPCM",
                    "adpcm_ea": "ADPCM",
                    "adpcm_ea_maxis_xa": "ADPCM",
                    "adpcm_ea_r1": "ADPCM",
                    "adpcm_ea_r2": "ADPCM",
                    "adpcm_ea_r3": "ADPCM",
                    "adpcm_ea_xas": "ADPCM",
                    "g722": "G.722)",
                    "g726": "G.726",
                    "g726le": "G.726",
                    "adpcm_ima_amv": "AMV",
                    "adpcm_ima_apc": "CRYO APC",
                    "adpcm_ima_dk3": "Duck DK3",
                    "adpcm_ima_dk4": "Duck DK4",
                    "adpcm_ima_ea_eacs": "EACS",
                    "adpcm_ima_ea_sead": "SEAD",
                    "adpcm_ima_iss": "Funcom ISS",
                    "adpcm_ima_oki": "Dialogic OKI",
                    "adpcm_ima_qt": "QuickTime",
                    "adpcm_ima_rad": "Radical",
                    "adpcm_ima_smjpeg": "Loki SDL MJPEG",
                    "adpcm_ima_wav": "WAV",
                    "adpcm_ima_ws": "Westwood",
                    "adpcm_ms": "Microsoft",
                    "adpcm_sbpro_2": "Sound Blaster Pro 2-bit",
                    "adpcm_sbpro_3": "Sound Blaster Pro 2.6-bit",
                    "adpcm_sbpro_4": "Sound Blaster Pro 4-bit",
                    "adpcm_swf": "Flash",
                    "adpcm_thp": "THP",
                    "adpcm_vima": "VIMA",
                    "vima": "VIMA)",
                    "adpcm_xa": "XA",
                    "adpcm_yamaha": "Yamaha",
                    "alac": "ALAC",
                    "amrnb": "AMR-NB",
                    "libopencore_amrnb": "AMR-NB",
                    "amrwb": "AMR-WB",
                    "libopencore_amrwb": "AMR-WB",
                    "ape": "Ape",
                    "atrac1": "ATRAC1",
                    "atrac3": "ATRAC3",
                    "atrac3plus": "ATRAC3+",
                    "on2avc": "AVC",
                    "binkaudio_dct": "Bink(DCT)",
                    "binkaudio_rdft": "Bink(RDFT)",
                    "bmv_audio": "BMV",
                    "comfortnoise": "RFC 3389 comfort noise generator",
                    "cook": "RealAudio G2",
                    "dsd_lsbf": "DSD",
                    "dsd_lsbf_planar": "DSD",
                    "dsd_msbf": "DSD",
                    "dsd_msbf_planar": "DSD",
                    "dsicinaudio": "CIN",
                    "dca": "DTS",
                    "eac3": "E-AC-3",
                    "evrc": "EVRC",
                    "flac": "FLAC",
                    "g723_1": "G.723.1",
                    "g729": "G.729",
                    "gsm": "GSM",
                    "libgsm": "gsm",
                    "gsm_ms": "GSM",
                    "libgsm_ms": "gsm_ms",
                    "iac": "IAC",
                    "libilbc": "iLBC",
                    "imc": "IMC",
                    "interplay_dpcm": "DPCM",
                    "mace3": "MACE 3:1",
                    "mace6": "MACE 6:1",
                    "metasound": "MetaSound",
                    "mlp": "MLP",
                    "mp1": "MP1",
                    "mp1float": "MP1",
                    "mp2": "MP2",
                    "mp2float": "MP2",
                    "mp3": "MP3",
                    "mp3float": "MP3",
                    "mp3adu": "ADU",
                    "mp3adufloat": "MP3ADU",
                    "mp3on4": "MP3onMP4",
                    "mp3on4float": "MP3onMP4",
                    "als": "MPEG-4als)",
                    "mpc7": "Musepack SV7",
                    "mpc8": "Musepack SV8",
                    "nellymoser": "Nellymoser Asao",
                    "opus": "Opus",
                    "libopus": "libopus Opus",
                    "paf_audio": "paf",
                    "pcm_alaw": "PCM A-law",
                    "pcm_bluray": "PCM Blu-ray",
                    "pcm_dvd": "PCM DVD",
                    "pcm_f32be": "PCM 32-bit floating point big-endian",
                    "pcm_f32le": "PCM 32-bit floating point little-endian",
                    "pcm_f64be": "PCM 64-bit floating point big-endian",
                    "pcm_f64le": "PCM 64-bit floating point little-endian",
                    "pcm_lxf": "PCM signed 20-bit little-endian planar",
                    "pcm_mulaw": "PCM mu-law",
                    "pcm_s16be": "PCM 16-bit",
                    "pcm_s16be_planar": "PCM 16-bit",
                    "pcm_s16le": "PCM 16-bit ",
                    "pcm_s16le_planar": "PCM 16-bit",
                    "pcm_s24be": "PCM 24-bit",
                    "pcm_s24daud": "PCM D-Cinema 24-bit",
                    "pcm_s24le": "PCM 24-bit",
                    "pcm_s24le_planar": "PCM 24-bit",
                    "pcm_s32be": "PCM 32-bit",
                    "pcm_s32le": "PCM 32-bit",
                    "pcm_s32le_planar": "PCM 32-bit",
                    "pcm_s8": "PCM signed 8-bit",
                    "pcm_s8_planar": "PCM 8-bit ",
                    "pcm_u16be": "PCM 16-bit",
                    "pcm_u16le": "PCM 16-bit",
                    "pcm_u24be": "PCM 24-bit",
                    "pcm_u24le": "PCM 24-bit",
                    "pcm_u32be": "PCM 32-bit",
                    "pcm_u32le": "PCM 32-bit",
                    "pcm_u8": "PCM 8-bit",
                    "pcm_zork": "PCM Zork",
                    "qcelp": "QCELP",
                    "qdm2": "QDesign Music Codec 2",
                    "real_144": "RealAudio 1.0",
                    "real_288": "RealAudio 2.0",
                    "ralf": "RealAudio Lossless",
                    "roq_dpcm": "DPCM id RoQ",
                    "s302m": "SMPTE 302M",
                    "shorten": "Shorten",
                    "sipr": "RealAudio SIPR",
                    "smackaud": "Smacker audio",
                    "sol_dpcm": "Sol",
                    "sonic": "Sonic",
                    "libspeex": "Speex ",
                    "tak": "TAK (Tom's lossless Audio Kompressor)",
                    "truehd": "TrueHD",
                    "truespeech": "TrueSpeech",
                    "tta": "True Audio",
                    "twinvq": "VQF TwinVQ",
                    "vmdaudio": "VMD",
                    "vorbis": "Vorbis",
                    "libvorbis": "Vorbis",
                    "wavesynth": "Wave synthesis pseudo-codec",
                    "wavpack": "WavPack",
                    "ws_snd1": "Westwood Audio (SND1)",
                    "wmalossless": "Windows Media Audio Lossless",
                    "wmapro": "Windows Media Audio 9 Professional",
                    "wmav1": "Windows Media Audio 1",
                    "wmav2": "Windows Media Audio 2",
                    "wmavoice": "Windows Media Audio Voice",
                    "xan_dpcm": "DPCM Xan"}

AUDIO_BIT_DEPTHS = {"u8": "8",
                    "s16": "16",
                    "s32": "24",
                    "flt": "32",
                    "dbl": "",
                    "u8p": "8",
                    "s16p": "16",
                    "s32p": "24",
                    "fltp": "32",
                    "dblp": ""}

class AudioMetadata(TimeBasedMetadata):
    __metaclass__ = ABCMeta
    AUDIO_CODECS = {"8svx_exp": "8SVX",
                    "8svx_fib": "8SVX",
                    "aac": "AAC",
                    "aac_latm": "AAC",
                    "ac3": "AC-3",
                    "ac3_fixed": "AC-3",
                    "adpcm_4xm": "ADPCM",
                    "adpcm_adx": "ADPCM",
                    "adpcm_afc": "ADPCM",
                    "adpcm_ct": "ADPCM",
                    "adpcm_dtk": "ADPCM",
                    "adpcm_ea": "ADPCM",
                    "adpcm_ea_maxis_xa": "ADPCM",
                    "adpcm_ea_r1": "ADPCM",
                    "adpcm_ea_r2": "ADPCM",
                    "adpcm_ea_r3": "ADPCM",
                    "adpcm_ea_xas": "ADPCM",
                    "g722": "G.722)",
                    "g726": "G.726",
                    "g726le": "G.726",
                    "adpcm_ima_amv": "AMV",
                    "adpcm_ima_apc": "CRYO APC",
                    "adpcm_ima_dk3": "Duck DK3",
                    "adpcm_ima_dk4": "Duck DK4",
                    "adpcm_ima_ea_eacs": "EACS",
                    "adpcm_ima_ea_sead": "SEAD",
                    "adpcm_ima_iss": "Funcom ISS",
                    "adpcm_ima_oki": "Dialogic OKI",
                    "adpcm_ima_qt": "QuickTime",
                    "adpcm_ima_rad": "Radical",
                    "adpcm_ima_smjpeg": "Loki SDL MJPEG",
                    "adpcm_ima_wav": "WAV",
                    "adpcm_ima_ws": "Westwood",
                    "adpcm_ms": "Microsoft",
                    "adpcm_sbpro_2": "Sound Blaster Pro 2-bit",
                    "adpcm_sbpro_3": "Sound Blaster Pro 2.6-bit",
                    "adpcm_sbpro_4": "Sound Blaster Pro 4-bit",
                    "adpcm_swf": "Flash",
                    "adpcm_thp": "THP",
                    "adpcm_vima": "VIMA",
                    "vima": "VIMA)",
                    "adpcm_xa": "XA",
                    "adpcm_yamaha": "Yamaha",
                    "alac": "ALAC",
                    "amrnb": "AMR-NB",
                    "libopencore_amrnb": "AMR-NB",
                    "amrwb": "AMR-WB",
                    "libopencore_amrwb": "AMR-WB",
                    "ape": "Ape",
                    "atrac1": "ATRAC1",
                    "atrac3": "ATRAC3",
                    "atrac3plus": "ATRAC3+",
                    "on2avc": "AVC",
                    "binkaudio_dct": "Bink(DCT)",
                    "binkaudio_rdft": "Bink(RDFT)",
                    "bmv_audio": "BMV",
                    "comfortnoise": "RFC 3389 comfort noise generator",
                    "cook": "RealAudio G2",
                    "dsd_lsbf": "DSD",
                    "dsd_lsbf_planar": "DSD",
                    "dsd_msbf": "DSD",
                    "dsd_msbf_planar": "DSD",
                    "dsicinaudio": "CIN",
                    "dca": "DTS",
                    "eac3": "E-AC-3",
                    "evrc": "EVRC",
                    "flac": "FLAC",
                    "g723_1": "G.723.1",
                    "g729": "G.729",
                    "gsm": "GSM",
                    "libgsm": "gsm",
                    "gsm_ms": "GSM",
                    "libgsm_ms": "gsm_ms",
                    "iac": "IAC",
                    "libilbc": "iLBC",
                    "imc": "IMC",
                    "interplay_dpcm": "DPCM",
                    "mace3": "MACE 3:1",
                    "mace6": "MACE 6:1",
                    "metasound": "MetaSound",
                    "mlp": "MLP",
                    "mp1": "MP1",
                    "mp1float": "MP1",
                    "mp2": "MP2",
                    "mp2float": "MP2",
                    "mp3": "MP3",
                    "mp3float": "MP3",
                    "mp3adu": "ADU",
                    "mp3adufloat": "MP3ADU",
                    "mp3on4": "MP3onMP4",
                    "mp3on4float": "MP3onMP4",
                    "als": "MPEG-4als)",
                    "mpc7": "Musepack SV7",
                    "mpc8": "Musepack SV8",
                    "nellymoser": "Nellymoser Asao",
                    "opus": "Opus",
                    "libopus": "libopus Opus",
                    "paf_audio": "paf",
                    "pcm_alaw": "PCM A-law",
                    "pcm_bluray": "PCM Blu-ray",
                    "pcm_dvd": "PCM DVD",
                    "pcm_f32be": "PCM 32-bit floating point big-endian",
                    "pcm_f32le": "PCM 32-bit floating point little-endian",
                    "pcm_f64be": "PCM 64-bit floating point big-endian",
                    "pcm_f64le": "PCM 64-bit floating point little-endian",
                    "pcm_lxf": "PCM signed 20-bit little-endian planar",
                    "pcm_mulaw": "PCM mu-law",
                    "pcm_s16be": "PCM 16-bit",
                    "pcm_s16be_planar": "PCM 16-bit",
                    "pcm_s16le": "PCM 16-bit ",
                    "pcm_s16le_planar": "PCM 16-bit",
                    "pcm_s24be": "PCM 24-bit",
                    "pcm_s24daud": "PCM D-Cinema 24-bit",
                    "pcm_s24le": "PCM 24-bit",
                    "pcm_s24le_planar": "PCM 24-bit",
                    "pcm_s32be": "PCM 32-bit",
                    "pcm_s32le": "PCM 32-bit",
                    "pcm_s32le_planar": "PCM 32-bit",
                    "pcm_s8": "PCM signed 8-bit",
                    "pcm_s8_planar": "PCM 8-bit ",
                    "pcm_u16be": "PCM 16-bit",
                    "pcm_u16le": "PCM 16-bit",
                    "pcm_u24be": "PCM 24-bit",
                    "pcm_u24le": "PCM 24-bit",
                    "pcm_u32be": "PCM 32-bit",
                    "pcm_u32le": "PCM 32-bit",
                    "pcm_u8": "PCM 8-bit",
                    "pcm_zork": "PCM Zork",
                    "qcelp": "QCELP",
                    "qdm2": "QDesign Music Codec 2",
                    "real_144": "RealAudio 1.0",
                    "real_288": "RealAudio 2.0",
                    "ralf": "RealAudio Lossless",
                    "roq_dpcm": "DPCM id RoQ",
                    "s302m": "SMPTE 302M",
                    "shorten": "Shorten",
                    "sipr": "RealAudio SIPR",
                    "smackaud": "Smacker audio",
                    "sol_dpcm": "Sol",
                    "sonic": "Sonic",
                    "libspeex": "Speex ",
                    "tak": "TAK (Tom's lossless Audio Kompressor)",
                    "truehd": "TrueHD",
                    "truespeech": "TrueSpeech",
                    "tta": "True Audio",
                    "twinvq": "VQF TwinVQ",
                    "vmdaudio": "VMD",
                    "vorbis": "Vorbis",
                    "libvorbis": "Vorbis",
                    "wavesynth": "Wave synthesis pseudo-codec",
                    "wavpack": "WavPack",
                    "ws_snd1": "Westwood Audio (SND1)",
                    "wmalossless": "Windows Media Audio Lossless",
                    "wmapro": "Windows Media Audio 9 Professional",
                    "wmav1": "Windows Media Audio 1",
                    "wmav2": "Windows Media Audio 2",
                    "wmavoice": "Windows Media Audio Voice",
                    "xan_dpcm": "DPCM Xan"}

    AUDIO_BIT_DEPTHS = {"u8": "8",
                        "s16": "16",
                        "s32": "24",
                        "flt": "32",
                        "dbl": "",
                        "u8p": "8",
                        "s16p": "16",
                        "s32p": "24",
                        "fltp": "32",
                        "dblp": ""}

    def __init__(self, filename):
        TimeBasedMetadata.__init__(self, filename)

    def __sizeofHuman(self, num):
        num = int(num)
        for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
            if num < 1024.0:
                return "%3.1f %s" % (num, x)
            num /= 1024.0

    @property
    def audioBitDepth(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
            if stream.getAttribute("codec_type") == "audio":
                return int(AUDIO_BIT_DEPTHS[stream.getAttribute("sample_fmt")])

    @property
    def audioChannels(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
                if stream.getAttribute("codec_type") == "audio":
                    return int(stream.getAttribute("channels"))

    @property
    def audioCodec(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
            if stream.getAttribute("codec_type") == "audio":
                return str(AUDIO_CODECS[stream.getAttribute("codec_name")])


    @property
    def audioCodecLongName(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
            if stream.getAttribute("codec_type") == "audio":
                return str(stream.getAttribute("codec_long_name"))

    @property
    def audioSampleRate(self):
        # TODO Make audioSampleRate Method
        for stream in self.xmlDom.getElementsByTagName('stream'):
            if stream.getAttribute("codec_type") == "audio":
                return int(stream.getAttribute("sample_rate"))

    @property
    def audioBitRate(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
            if stream.getAttribute("codec_type") == "audio":
                return int(stream.getAttribute("bit_rate"))

    @property
    def audioBitRateH(self):
        for stream in self.xmlDom.getElementsByTagName('stream'):
            if stream.getAttribute("codec_type") == "audio":
                return self.__sizeofHuman(int(stream.getAttribute("bit_rate")))



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import hashlib

from os.path import split, splitext, getctime, getsize, getmtime
from sys import stdout
import threading
from time import ctime, sleep
from abc import ABCMeta
import thread
import sys

VALID_VIDEO_FILE_EXTENSIONS = ['.avi', '.mov', '.mp4', '.mpeg', '.mpg', '.mkv']
VALID_AUDIO_FILE_EXTENSIONS = ['.wav', '.mp3', '.ogg']
VALID_IMAGE_FILE_EXTENSIONS = ['.bmp', '.gif', '.jpg', '.psd', '.tga', '.tif']
VALID_DOCUMENT_FILE_EXTENSIONS = ['.doc', '.docx', '.pdf', '.txt']
BUFFER = 8192

class FormatException(Exception):
    def __init__(self, value):
         self.value = value

    def __str__(self):
        return self.value

class MD5_Generator(threading.Thread):
    def __init__(self, file):
        threading.Thread.__init__(self)
        self.file = file
        self.total = getsize(file)
        self.isRunning = False
        self.completed = 0
        self.md5 = hashlib.md5()


    @property
    def running(self):
        return self.isRunning

    @property
    def progress(self):
        progress_lock = threading.Lock()
        with progress_lock:
            return self.completed

    @property
    def getChecksum(self):
        return self.md5.hexdigest()

    def run(self):

        self.isRunning = True
        with open(self.file, 'rb') as f:
            i = float(0)

            for chunk in iter(lambda: f.read(BUFFER), b''):
                i += BUFFER
                self.completed = int((i/self.total)*100)
                self.md5.update(chunk)

        self.isRunning = False



class FileMetadata(object):
    __metaclass__ = ABCMeta
    def __init__(self, sourcefile):
        self.___source = sourcefile
        self.___fileName = split(sourcefile)[1]
        self.___filePath = split(sourcefile)[0]
        self.___metadataDOM = None
        self._calculation_progress = 0
        self._calculation = None
        self._MD5 = None

    def __validateFileType(self):
        # check file extensions
        validation = True
        valid_extension = False
        file_extension = splitext(self.___source)[1]
        for extension in VALID_VIDEO_FILE_EXTENSIONS:
            if file_extension == extension:
                valid_extension = True
                break
        for extension in VALID_AUDIO_FILE_EXTENSIONS:
            if file_extension == extension:
                valid_extension = True
                break
        if not valid_extension:
            raise FormatException(file_extension + " is not an audio or video file.")
            # validation = False
        return validation



    def __sizeofHuman(self, num):
        num = int(num)
        for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
            if num < 1024.0:
                return "%3.1f %s" % (num, x)
            num /= 1024.0

    @property
    def isMD5Calculating(self):
        if self._calculation:
            return self._calculation.isAlive()
        else:
            return False

    @property
    def calulation_progresss(self):
        return self._calculation_progress

    @property
    def file_name(self):
        return self.___fileName

    @property
    def file_path(self):
        return self.___filePath

    @property
    def file_extension(self):
        return str(splitext(self.___source)[1])

    @property
    def format_type(self):
        #check if it's an audio format
        for extension in VALID_AUDIO_FILE_EXTENSIONS:
            if extension in self.___source:
                return "audio"

        #check if it's a video format
        for extension in VALID_VIDEO_FILE_EXTENSIONS:
            if extension in self.___source:
                return "video"
        return "unknown"

    @property
    def file_size(self):
        return getsize(self.___source)

    @property
    def file_size_human(self):
        return self.__sizeofHuman(getsize(self.___source))

    @property
    def date_created(self):
        return ctime(getctime(self.___source))

    @property
    def date_last_modified(self):
        return ctime(getmtime(self.___source))

    @property
    def MD5_hash(self):
        if self._MD5:
            return self._MD5
        elif self._calculation.isAlive:
            self._calculation.join()
            # self._MD5 = self.calculation
            return self._MD5
        else:
            raise Exception("MD5 not calculated yet")

    def getXML(self):
        pass


    def _calculate_MD5(self, progress):
        MD5 = hashlib.md5()
        # print "start"
        sleep(1)
        # total = self.file_size
        # checksum = MD5_Generator(self.___source, progress=True)
        checksum = MD5_Generator(self.___source)
        checksum.daemon = True
        checksum.start()
        # while checksum.running == True:
        while checksum.isRunning:
            # print checksum.running
            sleep(.25)
            self._calculation_progress = checksum.progress
            if progress == True:
                # print checksum.progress
                message = str(self._calculation_progress) + "%"
                # print(message),
                sys.stdout.write('\r' + message)
                sys.stdout.flush()
        checksum.join()
        if progress == True:
            sys.stdout.write("\r\033[K\r")
            sleep(.5)
            sys.stdout.flush()
        self._MD5 = checksum.getChecksum
        return self._MD5

    def calculate_MD5(self, progress=False, threaded=False):
        if threaded:
            self._calculation = threading.Thread(target=self._calculate_MD5, args=(progress,))
            self._calculation.daemon = True
            self._calculation.start()
        else:
            MD5 = self._calculate_MD5(progress)
            self._MD5 = MD5
            return MD5


    def calculate_SHA1(self, verbose=False):
        sha1 = hashlib.sha1()
        total = self.file_size

        with open(self.___source,'rb') as f:
            i = float(0)
            for chunk in iter(lambda: f.read(BUFFER), b''):
                if verbose:
                    i += BUFFER
                    completed = int((i/total)*100)
                    print str(completed) + "%"
                sha1.update(chunk)

        return sha1.hexdigest()

# TODO add a class for better progress reporting



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from onesheet import XMPMetadata
from abc import ABCMeta

class PDFMetadata(XMPMetadata):
    __metaclass__ = ABCMeta
    def __init__(self):
        self.___pdfKeywords = None
        self.___pdfPDFVersion = None
        self.___pdfProducer = None

    def getPdfKeywords(self):
        return self.___pdfKeywords

    def getPdfPDFVersion(self):
        return self.___pdfPDFVersion

    def getPdfProducer(self):
        return self.___pdfProducer




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from abc import ABCMeta
class EXFMetadata(object):
    __metaclass__ = ABCMeta
    def __init__(self):
        self.___exffExifVersion = None
        self.___exifFlashpixVersion = None
        self.___exifColorSpace = None
        self.___exifComponentsConfiguration = None
        self.___exifCompressedBitsPerPixel = None
        self.___exifPixelXDimension = None
        self.___exifPixelYDimension = None
        self.___exifUserComment = None
        self.___exifRelatedSoundFile = None
        self.___exifDateTimeOriginal = None
        self.___exifDateTimeDigitized = None
        self.___exifExposureTime = None
        self.___exifFNumber = None
        self.___exifExposureProgram = None
        self.___exifSpectralSensitivity = None
        self.___exifISOSpeedRatings = None
        self.___exifOECF = None
        self.___exifShutterSpeedValue = None
        self.___exifApertureValue = None
        self.___exifBrightnessValue = None
        self.___exifExposureBiasValue = None
        self.___exifMaxApertureValue = None
        self.___exifSubjectDistance = None
        self.___exifMeteringMode = None
        self.___exifLightSource = None
        self.___exifFlash = None
        self.___exifFocalLength = None
        self.___exifSubjectArea = None
        self.___exifFlashEnergy = None
        self.___exifSpacialFrequencyResponse = None
        self.___exifFocalPlaneXResolution = None
        self.___exifFocalPlaneYResolution = None
        self.___exifFocalPlaneResolutionUnit = None
        self.___exifSubjectLocation = None
        self.___exifExposureIndex = None
        self.___exifSensingMethod = None
        self.___exifFileSource = None
        self.___exifSceneType = None
        self.___exifCFAPattern = None
        self.___exifCustomRendered = None
        self.___exifExposureMode = None
        self.___exifWhiteBalance = None
        self.___exifDigitalZoomRatio = None
        self.___exifFocalLengthIn35mmFilm = None
        self.___exifSceneCaptureType = None
        self.___exifGainControl = None
        self.___exifContrast = None
        self.___exifSaturation = None
        self.___exifSharpness = None
        self.___exifDeviceSettingDescription = None
        self.___exifSubjectDistanceRange = None
        self.___exifImageUniqueID = None
        self.___exifGPSVersionID = None
        self.___exifGPSLatitude = None
        self.___exifGPSLongitude = None
        self.___exifGPSAltitudeRef = None
        self.___exifGPSAltitude = None
        self.___exifGPSTimeStamp = None
        self.___exifGPSSatelites = None
        self.___exifGPSStatus = None
        self.___exifMeasureMode = None
        self.___exifGPSDOP = None
        self.___exifGPSSpeedRef = None
        self.___exifGPSSpeed = None
        self.___exifGPSTrackRef = None
        self.___exifGPSTrack = None
        self.___exifGPSImgDirectionRef = None
        self.___exifGPSImgDirection = None
        self.___exifGPSMapDatum = None
        self.___exifGPSDestLatitude = None
        self.___exifGPSDestLongitude = None
        self.___exifGPSDestBearingRef = None
        self.___exifGPSDestBearing = None
        self.___exifGPSDestDistance = None
        self.___exifGPSProcessingMethod = None
        self.___exifGPSAreaInformation = None
        self.___exifGPSDifferential = None

    def getExifExifVersion(self):
        pass

    def getExifFlashpixVersion(self):
        return self.___exifFlashpixVersion

    def getExifColorSpace(self):
        return self.___exifColorSpace

    def getExifComponentsConfiguration(self):
        return self.___exifComponentsConfiguration

    def getExifCompressedBitsPerPixel(self):
        return self.___exifCompressedBitsPerPixel

    def getExifPixelXDimension(self):
        return self.___exifPixelXDimension

    def getExifPixelYDimension(self):
        return self.___exifPixelYDimension

    def getExifUserComment(self):
        return self.___exifUserComment

    def getExifRelatedSoundFile(self):
        return self.___exifRelatedSoundFile

    def getExifDateTimeOriginal(self):
        return self.___exifDateTimeOriginal

    def getExifDateTimeDigitized(self):
        return self.___exifDateTimeDigitized

    def getExifExposureTime(self):
        return self.___exifExposureTime

    def getExifFNumber(self):
        return self.___exifFNumber

    def getExifExposureProgram(self):
        return self.___exifExposureProgram

    def getExifSpectralSensitivity(self):
        return self.___exifSpectralSensitivity

    def getExifISOSpeedRatings(self):
        return self.___exifISOSpeedRatings

    def getExifOECF(self):
        return self.___exifOECF

    def getExifShutterSpeedValue(self):
        return self.___exifShutterSpeedValue

    def getExifApertureValue(self):
        return self.___exifApertureValue

    def getExifBrightnessValue(self):
        return self.___exifBrightnessValue

    def getExifExposureBiasValue(self):
        return self.___exifExposureBiasValue

    def getExifMaxApertureValue(self):
        return self.___exifMaxApertureValue

    def getExifSubjectDistance(self):
        return self.___exifSubjectDistance

    def getExifMeteringMode(self):
        return self.___exifMeteringMode

    def getExifLightSource(self):
        return self.___exifLightSource

    def getExifFlash(self):
        return self.___exifFlash

    def getExifFocalLength(self):
        return self.___exifFocalLength

    def getExifSubjectArea(self):
        return self.___exifSubjectArea

    def getExifFlashEnergy(self):
        return self.___exifFlashEnergy

    def getExifSpacialFrequencyResponse(self):
        return self.___exifSpacialFrequencyResponse

    def getExifFocalPlaneXResolution(self):
        return self.___exifFocalPlaneXResolution

    def getExifFocalPlaneYResolution(self):
        return self.___exifFocalPlaneYResolution

    def getExifFocalPlaneREsolutionUnit(self):
        pass

    def getExifSubjectLocation(self):
        return self.___exifSubjectLocation

    def getExifExposureIndex(self):
        return self.___exifExposureIndex

    def getExifSensingMethod(self):
        return self.___exifSensingMethod

    def getExifFileSource(self):
        return self.___exifFileSource

    def getExifSceneType(self):
        return self.___exifSceneType

    def getExifCFAPattern(self):
        return self.___exifCFAPattern

    def getExifCustomRendered(self):
        return self.___exifCustomRendered

    def getExifExposureMode(self):
        return self.___exifExposureMode

    def getExifWhiteBalance(self):
        return self.___exifWhiteBalance

    def getExifDigitalZoomRatio(self):
        return self.___exifDigitalZoomRatio

    def getExifFocalLengthIn35mmFilm(self):
        return self.___exifFocalLengthIn35mmFilm

    def getExifSceneCaptureType(self):
        return self.___exifSceneCaptureType

    def getExifGainControl(self):
        return self.___exifGainControl

    def getExifContrast(self):
        return self.___exifContrast

    def getExifSaturation(self):
        return self.___exifSaturation

    def getExifSharpness(self):
        return self.___exifSharpness

    def getExifDeviceSettingDescription(self):
        return self.___exifDeviceSettingDescription

    def getExifSubjectDistanceRange(self):
        return self.___exifSubjectDistanceRange

    def getExifImageUniqueID(self):
        return self.___exifImageUniqueID

    def getExifGPSVersionID(self):
        return self.___exifGPSVersionID

    def getExifGPSLatitude(self):
        return self.___exifGPSLatitude

    def getExifGPSLongitude(self):
        return self.___exifGPSLongitude

    def getExifGPSAltitudeRef(self):
        return self.___exifGPSAltitudeRef

    def getExifGPSAltitude(self):
        return self.___exifGPSAltitude

    def getExifGPSTimeStamp(self):
        return self.___exifGPSTimeStamp

    def getExifGPSSatelites(self):
        return self.___exifGPSSatelites

    def getExifGPSStatus(self):
        return self.___exifGPSStatus

    def getExifMeasureMode(self):
        return self.___exifMeasureMode

    def getExifGPSDOP(self):
        return self.___exifGPSDOP

    def getExifGPSSpeedRef(self):
        return self.___exifGPSSpeedRef

    def getExifGPSSpeed(self):
        return self.___exifGPSSpeed

    def getExifGPSTrackRef(self):
        return self.___exifGPSTrackRef

    def getExifGPSTrack(self):
        return self.___exifGPSTrack

    def getExifGPSImgDirectionRef(self):
        return self.___exifGPSImgDirectionRef

    def getExifGPSImgDirection(self):
        return self.___exifGPSImgDirection

    def getExifGPSMapDatum(self):
        return self.___exifGPSMapDatum

    def getExifGPSDestLatitude(self):
        return self.___exifGPSDestLatitude

    def getExifGPSDestLongitude(self):
        return self.___exifGPSDestLongitude

    def getExifGPSDestBearingRef(self):
        return self.___exifGPSDestBearingRef

    def getExifGPSDestBearing(self):
        return self.___exifGPSDestBearing

    def getExifGPSDestDistance(self):
        return self.___exifGPSDestDistance

    def getExifGPSProcessingMethod(self):
        return self.___exifGPSProcessingMethod

    def getExifGPSAreaInformation(self):
        return self.___exifGPSAreaInformation

    def getExifGPSDifferential(self):
        return self.___exifGPSDifferential

